package org.eclipse.cdt.rpm.editor;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.cdt.rpm.editor.preferences.PreferenceConstants;

public class RpmMacroProposalsList {

	private ArrayList list = new ArrayList();
	private String toStringStr;
	
	public RpmMacroProposalsList() {
		buildMacroList();
	}
	
	public void buildMacroList() {
		String macroProposalsPaths = Activator.getDefault().getPreferenceStore().getString(PreferenceConstants.P_MACRO_PROPOSALS_FILESPATH);
		String[] paths = macroProposalsPaths.split(";");
		for (int i = 0; i < paths.length; i++) {
			if (! paths[i].equals("")) {
				setMacrosList(paths[i]);
			}
		}
	}
	
	private void setMacrosList(String filename){
		String line = "";
		try {
			BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(filename)));
			line = reader.readLine();
			String key="", value="";
			while (line != null) {
				if (line.startsWith("%")) {
					String[] item = line.split("\t+| ", 2);
					try {
						/*
						 * Get values on more than one line
						 */
						if (line.trim().endsWith("\\")) {
							value = "\n";
							boolean isKeyLine  = true;
							while (line.trim().endsWith("\\")) {
								if (isKeyLine) {
									isKeyLine = false;
									key = item[0];
									if (item.length > 1)
										value += item[1].replace("\\", "\n\n");
								} else {
									value += line.substring(0, line.length() -1).trim() + "\n\t";
								}
								line = reader.readLine();
							}
						} else {
							key = item[0]; 
							value = item[1];							
						}
						item[0] = key.trim();
						item[1] = value.trim();
						list.add(item);
						toStringStr += item[0] + ": " + item[1] + "\n";
					} catch (Exception e) {
						line = reader.readLine();
						continue;
					}
					value = ""; key = "";
				}
				line = reader.readLine();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public List getMacroList() {
		return list;
	}

	public boolean findKey(String keyToFind) {
		Iterator iterator = list.iterator();
		String[] item;
		while (iterator.hasNext()) {
			item = (String[]) iterator.next();
			if (item[0].equals(keyToFind)) {
				return true;
			}
		}
		return false;
	}
	
	public List getProposals(String prefix) {
		Iterator iterator = list.iterator();
		List retList = new ArrayList(list.size());
		String[] item;
		int i = 0;
		while (iterator.hasNext()) {
			item = (String[]) iterator.next();
			if (item[0].startsWith(prefix)) {
				retList.add(item);
			}
			i++;
		}
		return retList;
	}
	
	public String getValue(String key) {
		Iterator iterator = list.iterator();
		String[] item;
		while (iterator.hasNext()) {
			item = (String[]) iterator.next();
			// to find key that begin with ? | % too
			if (item[0].endsWith(key) || item[0].endsWith(key.substring(1, key.length()))) {
				return item[1];
			}
		}
		return "";
	}

	public String toString() {
		return toStringStr;
	}

}
