package org.eclipse.cdt.rpm.editor;

import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.ITextDoubleClickStrategy;
import org.eclipse.jface.text.ITextHover;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.contentassist.ContentAssistant;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.reconciler.IReconciler;
import org.eclipse.jface.text.reconciler.MonoReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;

public class SpecfileConfiguration extends SourceViewerConfiguration {
	private SpecfileDoubleClickStrategy doubleClickStrategy;
//	private SpecfileTagScanner tagScanner;
	private SpecfileScanner scanner;
	private SpecfileChangelogScanner changelogScanner;
	private ColorManager colorManager;
	private SpecfileHover specfileHover;
	private SpecfileEditor editor;

	public SpecfileConfiguration(ColorManager colorManager, SpecfileEditor editor) {
		this.colorManager = colorManager;
		this.editor = editor;
	}
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return SpecfilePartitionScanner.SPEC_PARTITION_TYPES;
	}
	public ITextDoubleClickStrategy getDoubleClickStrategy(
		ISourceViewer sourceViewer,
		String contentType) {
		if (doubleClickStrategy == null)
			doubleClickStrategy = new SpecfileDoubleClickStrategy();
		return doubleClickStrategy;
	}

	protected SpecfileScanner getSpecfileScanner() {
		if (scanner == null) {
			scanner = new SpecfileScanner(colorManager);
			scanner.setDefaultReturnToken(
				new Token(
					new TextAttribute(
						colorManager.getColor(ISpecfileColorConstants.DEFAULT))));
		}
		return scanner;
	}
	
	protected SpecfileChangelogScanner getSpecfileChangelogScanner() {
		if  (changelogScanner == null) {
			changelogScanner = new SpecfileChangelogScanner(colorManager);
			changelogScanner.setDefaultReturnToken(
				new Token(
					new TextAttribute(
						colorManager.getColor(ISpecfileColorConstants.DEFAULT))));
		}
		return changelogScanner;
	}
	
	public ITextHover getTextHover(ISourceViewer sourceViewer, String contentType) {
		if (specfileHover == null)
			specfileHover = new SpecfileHover(this.editor);
		return specfileHover;
	}

	public IPresentationReconciler getPresentationReconciler(ISourceViewer sourceViewer) {
		PresentationReconciler reconciler = new PresentationReconciler();

		DefaultDamagerRepairer dr = new DefaultDamagerRepairer(getSpecfileScanner());
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);
//
//                dr = new DefaultDamagerRepairer(getSpecfileScanner());
//                reconciler.setDamager(dr, SpecfilePartitionScanner.SPEC_DEFAULT);
//                reconciler.setRepairer(dr, SpecfilePartitionScanner.SPEC_DEFAULT);
                
		dr = new DefaultDamagerRepairer(getSpecfileScanner());
		reconciler.setDamager(dr, SpecfilePartitionScanner.SPEC_SCRIPT);
		reconciler.setRepairer(dr, SpecfilePartitionScanner.SPEC_SCRIPT);
		
		dr = new DefaultDamagerRepairer(getSpecfileScanner());
		reconciler.setDamager(dr, SpecfilePartitionScanner.SPEC_FILES);
		reconciler.setRepairer(dr, SpecfilePartitionScanner.SPEC_FILES);
		
		dr = new DefaultDamagerRepairer(getSpecfileChangelogScanner());
		reconciler.setDamager(dr, SpecfilePartitionScanner.SPEC_CHANGELOG);
		reconciler.setRepairer(dr, SpecfilePartitionScanner.SPEC_CHANGELOG);

		return reconciler;
	}

	/*
	 * @see SourceViewerConfiguration#getReconciler(ISourceViewer)
	 */
	public IReconciler getReconciler(ISourceViewer sourceViewer) {
		SpecfileReconcilingStrategy strategy= new SpecfileReconcilingStrategy(editor);
		MonoReconciler reconciler= new MonoReconciler(strategy, false);
		reconciler.setProgressMonitor(new NullProgressMonitor());
		reconciler.setDelay(500);
		return reconciler;
	}
	
	/*
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getContentAssistant(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public IContentAssistant getContentAssistant(ISourceViewer sourceViewer) {
		ContentAssistant assistant= new ContentAssistant();
		IContentAssistProcessor processor= new SpecfileCompletionProcessor(editor);
		// add content assistance to all the supported contentType
		assistant.setContentAssistProcessor(processor, IDocument.DEFAULT_CONTENT_TYPE);
		assistant.setContentAssistProcessor(processor, SpecfilePartitionScanner.SPEC_SCRIPT);
		assistant.setContentAssistProcessor(processor,SpecfilePartitionScanner.SPEC_FILES);
		assistant.setContentAssistProcessor(processor,SpecfilePartitionScanner.SPEC_CHANGELOG);		
		// configure content assistance
		assistant.setContextInformationPopupOrientation(IContentAssistant.CONTEXT_INFO_ABOVE);
		assistant.setInformationControlCreator(getInformationControlCreator(sourceViewer));
		assistant.enableAutoInsert(true);
		return assistant;
	}
}