package org.eclipse.cdt.rpm.editor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.Token;

public class SpecfilePartitionScanner extends RuleBasedPartitionScanner {

	public final static String SPEC_SCRIPT = "__spec_script";
	public final static String SPEC_FILES = "__spec_files";
	public final static String SPEC_CHANGELOG = "__spec_changelog";
	
	public static String[] SPEC_PARTITION_TYPES = { IDocument.DEFAULT_CONTENT_TYPE, SPEC_SCRIPT,
			SPEC_FILES, SPEC_CHANGELOG };
	
	/** All possible headers for sections of the type SPEC_SCRIPT */
	private static String[] sectionHeaders = { "%prep", "%build", "%install", "%pre",
		"%preun", "%post", "%postun", "%clean"};

	/** All possible headers for section that can come after sections of the type SPEC_SCRIPT */
	private static String[] sectionEndingHeaders = { "%prep", "%build", "%install", "%pre",
		"%preun", "%post", "%postun", "%clean", "%files"};
	
	public SpecfilePartitionScanner() {
		// FIXME:  do we need this?
		super();
		
		IToken specScript = new Token(SPEC_SCRIPT);
		IToken specFiles = new Token(SPEC_FILES);
		IToken specChangelog = new Token(SPEC_CHANGELOG);

		List rules = new ArrayList();
		
		// %changelog
		rules.add(new MultiLineRule("%changelog", "", specChangelog, (char)0 , true));
		
		// "%prep", "%build", "%install", "%pre", "%preun", "%post", "%postun"
		for (int i = 0; i < sectionHeaders.length; i++)
			rules.add(new SectionRule(sectionHeaders[i], sectionEndingHeaders, specScript));

		// %files
		rules.add(new SectionRule("%files", new String[] { "%files",
				"%changelog" }, specFiles));
		
		IPredicateRule[] result= new IPredicateRule[rules.size()];
		rules.toArray(result);
		setPredicateRules(result);
	}
}
