package org.eclipse.cdt.rpm.editor;

import org.eclipse.cdt.rpm.editor.parser.Specfile;
import org.eclipse.cdt.rpm.editor.parser.SpecfileParser;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.reconciler.DirtyRegion;
import org.eclipse.jface.text.reconciler.IReconcilingStrategy;
import org.eclipse.jface.text.reconciler.IReconcilingStrategyExtension;

public class SpecfileReconcilingStrategy implements IReconcilingStrategy, IReconcilingStrategyExtension {

	private IDocument sDocument;
	private IProgressMonitor sProgressMonitor;
	private SpecfileParser sParser;
	private SpecfileFoldingStructureProvider sFoldingStructureProvider;

	public SpecfileReconcilingStrategy(SpecfileEditor editor) {
		sParser= new SpecfileParser();
		sFoldingStructureProvider= new SpecfileFoldingStructureProvider(editor);
	}

	public void setDocument(IDocument document) {
		sDocument= document;
		sFoldingStructureProvider.setDocument(sDocument);
	}

	public void setProgressMonitor(IProgressMonitor monitor) {
		sProgressMonitor= monitor;
		sFoldingStructureProvider.setProgressMonitor(sProgressMonitor);
	}

	public void reconcile(DirtyRegion dirtyRegion, IRegion subRegion) {
		reconcile();
	}

	public void reconcile(IRegion partition) {
		reconcile();
	}

	public void initialReconcile() {
		reconcile();
	}

	private void reconcile() {
		Specfile specfile= parseSpecfile();
		if (specfile != null) {
			updateEditor(specfile);
			updateFolding(specfile);
		}
	}

	private Specfile parseSpecfile() {
		return sParser.parse(sDocument);
	}

	private void updateEditor(final Specfile specfile) {
		return;
	}

	private void updateFolding(Specfile specfile) {
		sFoldingStructureProvider.updateFoldingRegions(specfile);
	}
}
